; HAM256 viewer v0.3

; HAM256 format devised by Andrew Owen
; Raster code by Simon Owen

org 57700
	jr		switch_on

	di						; switch interrupt mode
	im		1
	ei
	ret

	defb	"3"				; minor version number

org 57708					; 56 bytes
switch_on:
	di						; disable interrupts
	push	af				; save the registers
	push	bc
	push	de
	push	hl

	ld		de, 0xbfff		; D = register port, E = data port	
	ld		c, 0x3b			; C = ZXI base port
	xor		a				; count starts at zero
	
p_loop:
	ld		b, d			; register port
	out		(c), a			; palette entry to write
	ld		b, e			;
	ex		af, af'			;'save the count
	xor		a				; set the palette to all black
	out		(c), a
	ex		af, af'			;'restore the count
	inc		a				; next palette entry
	cp		64				; are we nearly there yet?
	jr		nz, p_loop		; repeat until all 64 have been done

	ld		b, d			; select register port
	out		(c), a			; A = 64 (mode group)
	ld		a, 1			; RGB mode (2 for HSL)
	ld		b, e			; data port
	out		(c), a			; enable RGB mode 

	ld		hl, screen		; bitmap and attributes
	ld		de, 0x4000		; screen address
	ld		bc, 6912		; bytes
	ldir					; block copy

	ld		a, 0x3b			; use ROM vector table
	ld		i, a			;

	pop		hl				; restore the registers
	pop		de
	pop		bc
	pop		af

enable_im2:	
	im		2				; switch it on
	ei						; enable interrupts
	ret						; return to BASIC

org 57764					; 80 bytes of code

im_main:
	push	af				; save registers
	push	bc
	push	de
	push	hl
	ex		af, af'			;'
	exx
	push	af
	push	bc
	push	de
	push	hl

	ld		hl, (0)			; padding		
	nop						;
	
	ld		hl, 473			; wait until just before the row
	
delay:
	dec		hl
	ld		a, h
	or		l
	jr		nz, delay		; wait until near top of main screen

	ld		hl, palette		; colour data
	ld		de, 0xbf00		; D = register select, E = data port (+1 due to OUTI)
	ld		c, 0x3b			; C = ZXI base port

	xor		a				; first CLUT entry is zero

	exx
	ld		de, 0x3f0c		; D = counter mask for 2 rows, E = 12 pairs of rows

loop1:
	ld		b, 16			; B=16 (x2 from below) entries to set
	
loop2:
	exx

	ld  	b, d			; register port
	out 	(c), a			; set CLUT
	ld  	b, e			; data port
	outi					; set colour
	inc		a               ; next CLUT entry

    ld		b, d			; same again...
	out		(c), a
	ld		b, e
	outi
	inc		a

	exx
	djnz	loop2          ; finish row

	and		d
	jr		nz, loop1        ; finish pair

pad_val:
	ld		b, 8			; timing for 48
	djnz 	$				; delay before next row

	nop						; 8 t-states
	nop
	
	dec		e
	jr		nz, loop1        ; finish screen

	pop		hl				; restore registers
	pop		de
	pop		bc
	pop		af
	ex		af, af'			;'
	exx
	pop		hl
	pop		de
	pop		bc
	pop		af
	
	jp		0x0038			; jump into ROM interrupt handler

org 57844
screen:
	incbin	"ham256.scr"

org 64756
palette:
	incbin	"ham256.pal"

org 65524
vector:
	jp		im_main			; jump to main interrupt handler

org 65535
	defb	24				; combined with first ROM byte forms JR vector

